package com.mdd.admin.socket.client;

import com.alibaba.fastjson2.JSONObject;
import com.mdd.admin.service.IUserConfigService;
import com.mdd.common.entity.UserConfig;
import com.mdd.common.util.JwtUtils;
import com.mdd.common.util.StringUtils;
import com.mdd.common.util.TimeUtils;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.concurrent.atomic.AtomicBoolean;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.enums.ReadyState;
import org.java_websocket.handshake.ServerHandshake;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Component
public class WebSocketService {
   @Resource
   IUserConfigService userConfigService;
   @Value("${like.websocketUrl}")
   private String websocketUrl;
   private WebSocketClient client;
   private Thread connectionThread;
   private AtomicBoolean isConnecting = new AtomicBoolean(false);

   @PostConstruct
   public void init() {
      UserConfig config = this.userConfigService.selectSystemConfig();
      if (config != null) {
         JSONObject baseSettings = JSONObject.parseObject(config.getValue()).getJSONObject("notice");
         if (baseSettings != null) {
            String timeZone = baseSettings.getString("timeZone");
            if (StringUtils.isNotEmpty(timeZone)) {
               TimeUtils.setTimeUtils(timeZone);
            }
         }
      }

      try {
         this.client = new MyWebSocketClient(new URI("ws://" + this.websocketUrl + "/websocket?type=0&session=" + JwtUtils.getToken((Integer)null, (Integer)null, (Integer)null)));
         this.connectionThread = new Thread(() -> {
            try {
               this.client.connectBlocking();
            } catch (InterruptedException var2) {
               Thread.currentThread().interrupt();
               this.client.close();
            }

         });
         this.connectionThread.start();
      } catch (URISyntaxException var4) {
         URISyntaxException e = var4;
         e.printStackTrace();
      }

   }

   private synchronized void reconnect(String message) {
      if (!this.isConnecting.getAndSet(true)) {
         if (this.connectionThread != null && this.connectionThread.isAlive()) {
            this.connectionThread.interrupt();
         }

         try {
            this.client.close();
            this.client = new MyWebSocketClient(new URI("ws://" + this.websocketUrl + "/websocket?type=0&session=" + JwtUtils.getToken((Integer)null, (Integer)null, (Integer)null)));
            this.connectionThread = new Thread(() -> {
               try {
                  this.client.connectBlocking();
                  if (this.client.isOpen() && StringUtils.isNotEmpty(message)) {
                     this.client.send(message);
                  }
               } catch (Exception var6) {
                  Exception e = var6;
                  e.printStackTrace();
               } finally {
                  this.isConnecting.set(false);
               }

            });
            this.connectionThread.start();
         } catch (Exception var3) {
            Exception e = var3;
            e.printStackTrace();
            this.isConnecting.set(false);
         }
      }

   }

   public void sendMessage(String message) {
      if (this.client != null && this.client.getReadyState() == ReadyState.OPEN) {
         if (StringUtils.isNotEmpty(message)) {
            this.client.send(message);
         }
      } else {
         this.reconnect(message);
         System.out.println("Attempting to reconnect and send message later...");
      }

   }

   @PreDestroy
   public void closeConnection() {
      if (this.client != null && !this.client.isClosed()) {
         this.client.close();
      }

      try {
         if (this.connectionThread != null && this.connectionThread.isAlive()) {
            this.connectionThread.join();
         }
      } catch (InterruptedException var2) {
         Thread.currentThread().interrupt();
      }

   }

   private static class MyWebSocketClient extends WebSocketClient {
      public MyWebSocketClient(URI serverUri) {
         super(serverUri);
      }

      public void onOpen(ServerHandshake handshakedata) {
         System.out.println("New connection opened");
      }

      public void onMessage(String message) {
         System.out.println("Received: " + message);
      }

      public void onClose(int code, String reason, boolean remote) {
         System.out.println("Connection closed by " + (remote ? "remote peer" : "us") + " Code: " + code + " Reason: " + reason);
      }

      public void onError(Exception ex) {
         ex.printStackTrace();
         this.close();
      }
   }
}
